use std::{fmt::Display, str::FromStr};

#[cfg(feature = "serde")]
use serde::{Deserialize, Serialize};

#[derive(Clone, Debug, PartialEq)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
pub struct Link<T: Display + PartialEq> {
    pub url: T,
    pub display: Option<T>,
}

impl<T> Display for Link<T>
where T: Display + PartialEq {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match &self.display {
            Some(d) => write!(f, "=> {} {d}", self.url),
            None => write!(f, "=> {}", self.url),
        }
    }
}

impl FromStr for Link<String> {
    type Err = super::Error;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        let Some(s) = s.strip_prefix("=> ") else {
            return Err(super::Error::MalformedLink);
        };
        if let Some((url, display)) = s.split_once(char::is_whitespace) {
            Ok(Self { url: url.to_string(), display: Some(display.to_string()) })
        } else {
            Ok(Self { url: s.to_string(), display: None })
        }
    }
}

impl TryFrom<&str> for Link<String> {
    type Error = super::Error;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        value.parse()
    }
}

impl<'a> TryFrom<&'a str> for Link<&'a str> {
    type Error = super::Error;

    fn try_from(value: &'a str) -> Result<Self, Self::Error> {
        let Some(s) = value.strip_prefix("=> ") else {
            return Err(super::Error::MalformedLink);
        };
        if let Some((url, display)) = s.split_once(char::is_whitespace) {
            Ok(Self { url, display: Some(display) })
        } else {
            Ok(Self { url: s , display: None })
        }
    }
}
